2010-08-23 Reini Urban <rurban@x-ray.at>

	* lib-src/winclient.c: Resolve xemacs symlink on cygwin.

2009-07-27 Reini Urban <rurban@x-ray.at>

	* lib-src/winclient.c: Support cygwin paths. Add longer STARTUP_DELAY 
        and CONNECT_RETRIES due to long GUI and DDE initialization on cygwin.

diff -u  lib-src/Makefile.in.in.orig
--- lib-src/Makefile.in.in.orig	2007-05-21 05:50:19.000000000 +0200
+++ lib-src/Makefile.in.in	2009-06-24 09:40:30.000000000 +0200
@@ -375,7 +375,7 @@
 	$(CC) $(cflags) ${srcdir}/../nt/minitar.c $(ldflags) -lz -o $@
 
 winclient: ${srcdir}/winclient.c
-	$(CC) $(cflags) ${srcdir}/winclient.c $(ldflags) -o $@
+	$(CC) $(cflags) -mwindows ${srcdir}/winclient.c $(ldflags) -o $@
 
 hexl: ${srcdir}/hexl.c
 	$(CC) $(cflags) ${srcdir}/hexl.c $(ldflags) -o $@
diff -u  lib-src/winclient.c.orig
--- lib-src/winclient.c.orig	2009-06-24 09:30:22.000000000 +0200
+++ lib-src/winclient.c	2010-08-23 10:12:41.351524800 +0200
@@ -1,5 +1,6 @@
 /* DDE client for XEmacs.
    Copyright (C) 2002 Alastair J. Houghton
+   Copyright (C) 2009,2010 Reini Urban
 
    This file is part of XEmacs.
 
@@ -31,6 +32,9 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
+#ifdef __CYGWIN__
+#include <sys/cygwin.h>
+#endif
 
 static void error (const char* s1, const char* s2);
 static void fatal (const char *s1, const char *s2);
@@ -40,6 +44,8 @@
 /* -- Post-Include Defines -------------------------------------------------- */
 
 /* Timeouts & delays */
+#define STARTUP_DELAY           10000 	        /* ms */
+#define CONNECT_RETRIES         10
 #define CONNECT_DELAY		500		/* ms */
 #define TRANSACTION_TIMEOUT	5000		/* ms */
 #define MAX_INPUT_IDLE_WAIT     INFINITE	/* ms */
@@ -51,7 +57,7 @@
 
 /* XEmacs program name */
 #ifdef __CYGWIN__
-#define PROGRAM_TO_RUN	"xemacs-21.5-b28.exe"
+#define PROGRAM_TO_RUN	"xemacs"
 #else
 #define PROGRAM_TO_RUN	"xemacs.exe"
 #endif
@@ -191,9 +197,40 @@
       int                 n;
       
       /* Try to start the program */
+#ifdef __CYGWIN__
+      FILE*               output; 
+      char                tmp[MAX_PATH];
+      char                program[MAX_PATH];
+      int                 len;
+
+      /* find xemacs */
+      output = popen("/usr/bin/which "PROGRAM_TO_RUN, "r"); 
+      fgets(program, MAX_PATH-1, output);
+      len = strlen(program);
+      if (len) program[len-1] = 0; 	/* strip ending \n */
+      else strcpy(program, PROGRAM_TO_RUN);
+# ifdef DEBUG
+      fprintf(stderr, "DEBUG openConversation found \"%s\"\n", program);
+# endif
+
+      strcpy(tmp, "cygpath -wa ");
+      strcat(tmp, program);
+      /* resolve the symlink /usr/bin/xemacs and convert to winpath */
+      output = popen(tmp, "r"); 
+      fgets(program, MAX_PATH-1, output);
+      len = strlen(program);
+      if (len) program[len-1] = 0;	/* strip ending \n */
+      else strcpy(program, PROGRAM_TO_RUN);
+# ifdef DEBUG
+      fprintf(stderr, "DEBUG => \"%s\"\n", program);
+# endif
+#else
+      char 		  program[] = PROGRAM_TO_RUN;
+#endif
+
       ZeroMemory (&sti, sizeof (sti));
       sti.cb = sizeof (sti);
-      if (!CreateProcess (NULL, PROGRAM_TO_RUN, NULL, NULL, FALSE, 0,
+      if (!CreateProcess (NULL, program, NULL, NULL, FALSE, 0,
 			  NULL, NULL, &sti, &pi))
 	{
 	  MessageBox (NULL, "Could not start process.",
@@ -205,12 +242,15 @@
       /* Wait for the process to enter an idle state */
       WaitForInputIdle (pi.hProcess, MAX_INPUT_IDLE_WAIT);
 
+      /* Wait longer to let the process init the GUI and initialize the DDE server */
+      Sleep (STARTUP_DELAY);
+
       /* Close the handles */
       CloseHandle (pi.hThread);
       CloseHandle (pi.hProcess);
       
-      /* Try to connect */
-      for (n = 0; n < 5; n++)
+      /* Try to connect. Process startup and XEmacs init might be slow */
+      for (n = 0; n < CONNECT_RETRIES; n++)
 	{
 	  Sleep (CONNECT_DELAY);
 	  
@@ -272,6 +312,29 @@
   char            *buf = NULL;
   unsigned        len;
   
+  /* On Cygwin convert the path - first part - back to POSIX */
+#ifdef __CYGWIN__
+  int             err;
+  int             pathlen;
+  char            *fullpath;
+  const int       extra = 260 + 1001;
+
+  pathlen = strlen(lpszFileName1) + extra;
+  fullpath = (char *) xmalloc (pathlen);
+  err = cygwin_conv_path(CCP_WIN_A_TO_POSIX | CCP_RELATIVE,
+                         lpszFileName1, fullpath, pathlen);
+# ifdef DEBUG
+  fprintf(stderr, "DEBUG doFile(hConv, \"%s\", \"%s\")\n", lpszFileName1, lpszFileName2);
+  fprintf(stderr, "DEBUG W2P \"%s\" => \"%s\" err=%d\n", lpszFileName1, fullpath, err);
+# endif
+  if (!err) {
+      strcpy(lpszFileName1, fullpath);
+      len = strlen(lpszFileName1);
+      if (strlen(lpszFileName2) && (lpszFileName1[len-1] != '/'))
+          strcat(lpszFileName1, "/");
+  }
+#endif
+
   /* Calculate the buffer length */
   len = strlen (lpszFileName1) + strlen (lpszFileName2)
     + strlen (COMMAND_FORMAT);
@@ -296,6 +359,13 @@
   DdeClientTransaction (buf, len, hConv, NULL,
 			0, XTYP_EXECUTE, TRANSACTION_TIMEOUT, NULL);
 
+#ifdef __CYGWIN__
+# ifdef DEBUG
+  fprintf(stderr, "DEBUG DDE => %s\n", buf);
+# endif
+  if (!err)
+      free (fullpath);
+#endif
   free (buf);
   
   return 0;
@@ -404,13 +474,20 @@
   int             ret = 0;
   HANDLE          hFindFile = NULL;
   WIN32_FIND_DATA wfd;
+#ifdef __CYGWIN__
+  int             err;
+#endif
 
   /* Retrieve arguments */
   while ((arg = getNextArg ((const char**)&lpszCommandLine, &len)) != NULL)
     {
       /* First find the canonical path name */
       fullpath = filepart = NULL;
+#ifdef __CYGWIN__
+      pathlen = len + 260 + 1001;
+#else
       pathlen = GetFullPathName (arg, 0, fullpath, &filepart);
+#endif
 
       fullpath = (char *) xmalloc (pathlen);
 
@@ -425,6 +502,17 @@
 	  break;
 	}
 
+#ifdef __CYGWIN__
+      err = cygwin_conv_path(CCP_POSIX_TO_WIN_A | CCP_RELATIVE, arg, fullpath, pathlen);
+# ifdef DEBUG
+      fprintf(stderr, "DEBUG P2W \"%s\" => \"%s\" err=%d\n", arg, fullpath, err);
+# endif
+      if (err)
+          ret = doFile (hConv, "", arg);
+      else
+          arg = fullpath;
+#endif
+
       GetFullPathName (arg, pathlen, fullpath, &filepart);
 
       /* Find the first matching file */
@@ -454,7 +542,10 @@
 
       /* Release the path name buffers */
       free (fullpath);
-      free (arg);
+#ifdef __CYGWIN__
+      if (err)
+          free (arg);
+#endif
 
       if (ret)
 	break;
